
import fr.emn.spiraclock.*;
import java.util.*;
import java.awt.*;
import java.awt.event.*;
import fr.emn.utils.file.*;
import javax.swing.*;
import java.beans.*;

public class ExampleSchedule extends Schedule implements ActionListener {

  /**
   * local Schedule properties. Automatically loaded and saved by the SpiraClock engine along sessions
   *  with the methods getProperties() and setProperties(Hashtable props).
   **/
  private Hashtable properties = new Hashtable();
  /**
   * an example of local property management.
   */
  private boolean exampleProp = true;

  public ExampleSchedule() {
  }

  /**
   * Called by SpiraClock engine to manage local properties persistence along sessions (events colors, etc...).
   * @return local Schedule properties.
   */
  public Hashtable getProperties() {
    //updating local properties
    properties.put("EXPROP", new Boolean(exampleProp));
    //return updated properties
    return properties;//return null if there is no need to manage local properties
  }

  /**
   * Called by SpiraClock engine to manage local properties.
   * After application loading, SpiraClock give the loaded properties (from the SpiraClock.ini file)
   *  to the Schedule, allowing local properties extraction.
   * @param props the properties loaded from the file
   */
  public void setProperties(Hashtable props) {
    //extracting local property from the global properties set
    Boolean tmp = (Boolean) props.get("EXPROP");

    if (tmp != null)// if the local property exists in the ini file
      exampleProp = tmp.booleanValue();// read it
    else// local property doesn't exist (may be it's the first running of this schedule)
      props.put("EXPROP", new Boolean(exampleProp));// create the property entry in global props.
    properties.put("EXPROP", new Boolean(exampleProp));// put the property entry in the local props.
  }

  public String getName() {
    //return this schedule name.
    return "SpiraClock SDK Exemple Schedule";
  }


  public void beforeClosing() {
    //stop console message redirection to a file
    SaveOutput.stop();
  }

  /**
   * subclass this method to reload all your data (from a file, or another app...)
   */
  public void reload() {
  }

  /**
   * Subclass this method to return all the appointments of your schedule for ther given day
   * @param day
   * @return
   */
  public Appointment[] getDayAppointments(ATime day) {
    //return all appointments for the given day...
    //for example, we will create 2 appointments for this day
    //normally, this method will dynamically return appointments according to the schedule
    //behaviour (see EMNBusSchedule.java for example). OutClock, for example,
    //will gather in Outlook and return all the events for the given day

    //setting appointment properties
    AppProperties appProps = new AppProperties();
    //setting appointment category
    appProps.setCategory("test category");
    //setting appointment display color
    appProps.setColor(Color.BLUE);
    //setting appointment fade on end property
    appProps.setFadeOnEnd(true);
    //setting appointment overlapping
    appProps.setOverlap(false);
    //setting appointment show end properety (display a small time amount after appointment occurs)
    appProps.setShowEnd(false);
    //setting appointment show start properety (display a small time amount before appointment occurs)
    appProps.setShowStart(false);

    // appointment start
    ATime start = new ATime(day.year, day.dayOfYear, 12, 0, 0);
    //appointment end
    ATime end = new ATime(day.year, day.dayOfYear, 14, 30, 0);
    //subject
    String subject = "test appointment subject";
    //body
    String body = "this appointment is only for ExempleSchedule test.";
    //creating appointment object
    Appointment a1 = new Appointment(start, end, subject, body, appProps);

    //another appointment
    appProps = new AppProperties();
    appProps.setCategory("meeting");
    appProps.setColor(Color.RED);
    appProps.setFadeOnEnd(true);
    appProps.setOverlap(false);
    appProps.setShowEnd(true);
    appProps.setShowStart(true);
    start = new ATime(day.year, day.dayOfYear, 17, 0, 0);
    end = new ATime(day.year, day.dayOfYear, 17, 30, 0);
    subject = "Lab. council";
    body = "Meeting with lab. staff";
    Appointment a2 = new Appointment(start, end, subject, body, appProps);

    //return the 2 appointments in an Appointment array
    return new Appointment[] {a1, a2};
  }

  /**
   * This method, called by the SpiraClock engine, allows you to install entries in the SpiraClock popup menu.
   *  (useful allow user setting schedule properties
   * @param popup the JPopupMenu object where to add your JMenuItem
   */
  public void installPopupCommands(JPopupMenu popup) {

    popup.add(new JMenuItem("Set example property") {
      {
        addActionListener(ExampleSchedule.this);
      }
    });
    JMenuItem nested = new JMenu("Nested menus example");
    nested.add(new JMenuItem("reload...") {
      {
        addActionListener(ExampleSchedule.this);
      }
    });
    popup.add(nested);
  }

  /**
   * Occurs when an action is performed
   * @param e
   */
  public void actionPerformed(ActionEvent e) {
    String command = e.getActionCommand();
    if (command.equals("Set example property")) {
      setExampleProperty();
      fireStateChanged();
    }
    if (command.equals("reload...")) {
      reload();
    }
  }

  /**
   * Shows a dialog pane to set a boolean property
   */
  private void setExampleProperty() {
    final JDialog dialog = new JDialog();
    dialog.setTitle("Configure Example property");
    Object[] options = {"OK", "CANCEL"};
    final JOptionPane pane = new JOptionPane(null, JOptionPane.DEFAULT_OPTION, JOptionPane.OK_CANCEL_OPTION, null,
                                             options, options[0]);
    JPanel paneContent = new JPanel();
    paneContent.setLayout(new BoxLayout(paneContent, BoxLayout.Y_AXIS));
    final JCheckBox checkbox = new JCheckBox("Example Property", exampleProp);
    paneContent.add(checkbox);
    pane.add(paneContent, 1);
    pane.addPropertyChangeListener(new PropertyChangeListener() {
      public void propertyChange(PropertyChangeEvent e) {
        String prop = e.getPropertyName();
        if (dialog.isVisible() && (e.getSource() == pane) && (prop.equals(JOptionPane.VALUE_PROPERTY) || prop.equals(JOptionPane.INPUT_VALUE_PROPERTY))) {
          Object value = pane.getValue();
          if (value == JOptionPane.UNINITIALIZED_VALUE) {
            //ignore reset
            return;
          }
          // Reset the JOptionPane's value.
          pane.setValue(JOptionPane.UNINITIALIZED_VALUE);
          if (value.equals("OK")) {
            //changes accepted, so update properties
            exampleProp = checkbox.isSelected();
            //unshow the dialog
            dialog.setVisible(false);
            //redraw all app frames

          } else { // user closed dialog or clicked cancel
            dialog.setVisible(false);
          }
        }
      }
    });
    dialog.setContentPane(pane);
    //dialog.setResizable(false);
    dialog.setModal(true);
    dialog.pack();
    Dimension screen = Toolkit.getDefaultToolkit().getScreenSize();
    dialog.setLocation((int)(screen.getWidth() - dialog.getSize().getWidth()) / 2, (int)(screen.getHeight() - dialog.getHeight()) / 2);
    dialog.setVisible(true);
  }


//main method to launch the SpiraClock application
  public static void main(String[] args) {
    try{
      //following code is optionnal... use it if you want to redirect console messages in a
      // file to gather errors informations
      javax.swing.UIManager.setLookAndFeel(javax.swing.UIManager.getSystemLookAndFeelClassName());
      Calendar c = Calendar.getInstance();
      c.clear();
      c.setTime(new Date(System.currentTimeMillis()));
      String errdate = String.valueOf(c.get(Calendar.DAY_OF_YEAR))+"-"+String.valueOf(c.get(Calendar.YEAR))+"-"+String.valueOf(c.get(Calendar.HOUR_OF_DAY))+"h"+String.valueOf(c.get(Calendar.MINUTE)+"m"+String.valueOf(c.get(Calendar.SECOND)));
      SaveOutput.start("Example_err.log");
      System.out.println("Example v1.0-SpiraClock v1.1e log file");
      System.out.println();
      System.out.println(c.getTime());
      System.out.println();
      System.out.print(System.getProperty("java.vm.name") + " version " + System.getProperty("java.vm.version") + " by " + System.getProperty("java.vm.vendor"));
      System.out.println();
      System.out.println();

      //create the schedule
      ExampleSchedule sc = new ExampleSchedule();
      //create the SpiraClock frame
      SpiraWindow frame = new SpiraWindow(sc);
      //show SpiraClock
      frame.setVisible(true);
   }
     catch (Exception e) {
      System.out.println();
      System.out.println(e.getClass().getName() + " message:");
      System.out.println(e.getMessage());
      System.out.println();
      System.out.println(e.getClass().getName() + " Stack Trace:");
      e.printStackTrace();
      SaveOutput.stop();
      System.exit(0);
     }

  }

}
